/**
  *  \file client/dialogs/shipbuilddlg.h
  *  \brief WShipBuildDialog dialog base
  *
  *  (c) 2001-2022 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_DIALOGS_SHIPBUILDDLG_H
#define PCC_V2_CLIENT_DIALOGS_SHIPBUILDDLG_H

#include "client/wcomplist.h"
#include "client/wcost.h"
#include "client/widgets/buildorderdisplay.h"
#include "client/widgets/engineinfo.h"
#include "client/widgets/enginestorage.h"
#include "client/widgets/partcount.h"
#include "client/widgets/partstorage.h"
#include "client/winfo.h"
#include "game/actions/baseship.h"
#include "ui/ui.h"
#include "ui/widgets/cardgroup.h"
#include "ui/widgets/cardtab.h"

/** \class WShipBuildDialog
    \brief Ship Build Dialog, general version

    This class provides a complete ship build screen which allows the
    user to select a ship type. It does not provide buttons to confirm
    a build transaction. A derived class must add these. */
class WShipBuildDialog : public UIWindow {
    WHullListbox      hull_list;
    WEngineListbox    engine_list;
    WBeamListbox      beam_list;
    WLauncherListbox  torp_list;

    WHullInfo         hull_info;
    WBeamInfo         beam_info;
    WTorpInfo         torp_info;
    WEngineInfo       engine_info;

    WPartStorageGroup hull_storage, torp_storage, beam_storage;
    WEngineStorageInfo engine_storage;
    WEngineInfoDiagram engine_diag;

    WPartCountGroup beam_count, torp_count;

    bool withPart;

    void init(std::vector<int>& hulls);

 protected:
    GStarbaseBuildShipAction  action;

    void onPlanetChanged();

 private:
    WItemCostDisplay  item_cost_dpy;
    WBuildOrderDisplay order_dpy;

    UICardGroup       cards;
    UICardGroupTab    tabs;

    SignalConnection conn_update;

 protected:
    // Must be declared exactly here so that it dies first.
    ObjectHolder      h;

    void updateForNewHull(bool initial);
    void updateHullPage();
    void updateBeamPage();
    void updateEnginePage();
    void updateLauncherPage();
    void updateBuildOrder();
 public:
    WShipBuildDialog(string_t name, GPlanet& planet, bool withPart, std::vector<int>& hulls);
    ~WShipBuildDialog();

    bool handleEvent(const UIEvent& event, bool second_pass);

    // public because not all compilers allow it to be protected when taking a member pointer
    void onDetailedBill();

    WHullListbox& getHullList();
    void handleHullListUpdate();
    void setBuildOrder(const TBuildOrder& order);
};

#endif
