/**
  *  \file client/widgets/ammoinfo.cc
  *
  *  (c) 2001-2022 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/widgets/ammoinfo.h"
#include "gfx/gfxcontext.h"
#include "util/nls.h"
#include "cpluslib/format.h"
#include "ui/skin.h"
#include "ui/uiglobal.h"
#include "util/misc.h"

/** Constructor.
    \param transfer Container to observe
    \param observe Cargo type to observe (can be changed with setCargoType). */
WAmmoInfo::WAmmoInfo(GCargoContainer& transfer, GCargoType observe)
    : transfer(transfer), observe(observe),
      btn_plus("+", '+'), btn_minus("-", '-')
{
    conn = UIBaseWidget::observe(transfer.sig_changed);
    addChildWidget(btn_plus, 0);
    addChildWidget(btn_minus, 0);
}

WAmmoInfo::~WAmmoInfo()
{ }

void
WAmmoInfo::drawContent(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    int y      = getExtent().y;
    int32 have = transfer.getCargo(observe);
    int32 max  = transfer.getMaxCargo(observe);

    /* Line 1: "Blafasel Starbase" */
    ctx.setColor(UIColor::tc_Static);
    ctx.useFont(*fonts[FONT_TITLE]);
    outTextF(ctx, getExtent().x, y, getExtent().w-20, transfer.getName());
    y += font_heights[FONT_TITLE] + 4;

    /* Line 2: "You have %d <zomething>" */
    ctx.setSolidBackground();
    ctx.useFont(*fonts[FONT_NORMAL]);
    outTextF(ctx, getExtent().x, y, getExtent().w-20,
             format(observe == el_Fighters
                    ? _("You have %d fighter%!1{s%}.")
                    : _("You have %d torpedo%!1{es%}."),
                    numToString(have)));
    y += font_heights[FONT_NORMAL];

    /* Line 3: "You have %d left" */
    string_t expl;
    if (have >= max) {
        expl = _("This is the maximum amount.");
    } else if (max - have <= 5000) {
        /* the average player has much fewer torps on a starbase. let
           them rest in peace not knowing there's a limit, most of the
           time. */
        expl = format(_("You have room for %d more."), numToString(max - have));
    }
    outTextF(ctx, getExtent().x, y, getExtent().w-20, expl);

    can.update(getExtent());
}

void
WAmmoInfo::getLayoutInfo(LayoutInfo& li)
{
    li.flags = 0;
    li.pref_size = li.min_size =
        font_sizes[FONT_NORMAL].scaledBy(20, 2).movedBy(0, font_heights[FONT_TITLE] + 5);
}

void
WAmmoInfo::setCargoType(GCargoType observe)
{
    this->observe = observe;
    drawWidget();
}

void
WAmmoInfo::onResize(GfxRect /*oldsize*/, bool /*by_layout*/)
{
    GfxRect x(0, 0, 20, 20);
    x.moveToEdge(getExtent(), 2, 0, 0);
    btn_plus.setExtent(x);
    x.moveBy(0, 25);
    btn_minus.setExtent(x);
}
