/**
  *  \file ui/widgets/timer.h
  *  \brief Timer Widget
  *
  *  (c) 2001-2022 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WIDGETS_TIMER_H
#define PCC_V2_UI_WIDGETS_TIMER_H

#include "ui/ui.h"
#include "cpluslib/sigslot.h"

/** Timer. A helper widget providing periodic or single-shot timers
    derived from ev_Tick events. Just add it to some widget and make
    sure it is part of regular event distribution. The timer raises a
    signal sig_tick when it reaches its count. */
class UITimer : public UIWidget {
    int this_timer;
    int next_timer;
    int counter;

 public:
    UITimer();
    ~UITimer();

    void startSingleShot(int msecs);
    void startPeriodic(int msecs);

    bool handleEvent(const UIEvent& event, bool second_pass);
    void getLayoutInfo(LayoutInfo& li);
    void drawContent(GfxCanvas& can);

    int getCounter() const;

    Signal0 sig_tick;
};


/** Get number of times this timer fired since its last start. (using
    startSingleShot, startPeriodic). Each start resets the counter to
    zero. With a single-shot timer, the result is zero if the timer
    runs, one if it fired. With a periodic timer, it counts up for
    each tick. */
inline int
UITimer::getCounter() const
{
    return counter;
}

#endif
