/**
  *  \file ui/widgets/tree.h
  *  \brief Tree widget
  *
  *  (c) 2001-2022 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WIDGETS_TREE_H
#define PCC_V2_UI_WIDGETS_TREE_H

#include <vector>
#include "cpluslib/types.h"
#include "cpluslib/string.h"
#include "ui/listbox.h"
#include "cpluslib/ptrvector.h"

/** Tree widget. Displays a foldable tree of text items. */
class UITreeWidget : public UIListbox {
 public:
    /* Representation of the tree. See addItem. */
    struct Item {
        int32_t id;
        int32_t level;
        bool open;
        string_t label;
    };

 private:
    ptr_vector<Item> items;
    std::vector<uint32_t> lineToItem;

    uint32_t getParentOf(uint32_t itemIndex);
    uint32_t getLineOf(uint32_t itemIndex);

 public:
    UITreeWidget(int id, int lines, int width);
    ~UITreeWidget();

    void addItem(int32_t id, int32_t level, string_t label);
    void addItem(int32_t id, int32_t level, string_t label, bool open);
    void updateAfterModification();

    void drawPart(GfxCanvas& can, int from, int to);
    virtual void drawItem(GfxContext& ctx, GfxRect area, const Item& it);
    bool handleEvent(const UIEvent& event, bool second_pass);
    void onItemClick(int item, int x);

    uint32_t getCurrentItemIndex() const;
    uint32_t getItemIndex(int line) const;
    int32_t getItemId(uint32_t itemIndex) const;
    bool hasChildren(uint32_t itemIndex) const;
    void toggleItem(uint32_t itemIndex);
};

#endif
